import phm, time, math
from Tkinter import *
              
data = []
index = 0

func_list = ['r2rtime', 'r2ftime', 'f2rtime', 'f2ftime', 'set2rtime', \
'set2ftime', 'clr2rtime', 'clr2ftime','pulse2rtime', 'pulse2ftime',\
'multi_r2rtime', 'pendulum_period']
src_list = ['D0', 'D1', 'D2', 'D3', 'CMP']
dst_list = ['D0', 'D1', 'D2', 'D3', 'CMP']

def hist():		# making Histograms from time measurement data
    global data
    if len(data) < 2:
        return

    min = 2000000L
    max = 0L
    for xy in data:
        if xy[1] < min:
            min = xy[1]
        if xy[1] > max:
            max = xy[1]
    span = max - min
    
    fd.remove_lines()
    fd.remove_boxes()
    if span < 10:			# only one value per HM channel
        numbins = int(span) + 1
        hm = [0.] * numbins
        for d in data:			#make histogram
            k = int(d[1]-min)
            hm[k] = hm[k] + 1      
        peak = 0			# find the peak value
        for h in hm:
            if h > peak:
                peak = h
        fd.set_scale(min-1,0,max+2,peak)
#        print 'scale = ', min,' ',max,' ', peak
        for i in range(numbins):	#make the line to plot it
            xy = []
            xy.append((min+i, 0))
            xy.append((min+i+1, hm[i]))
            fd.box(xy)
    else:			# Large range, need to group values
        numbins = 10
        hm = [0.] * numbins
        max = float(max)
        min = float(min)
        binsize = (max - min) / numbins
        for i in range(numbins):
            a1 = min + binsize * i
            a2 = min + binsize * (i+1)
            for xy in data:
                if a1 <= xy[1] < a2:
                    hm[i] = hm[i] + 1
        peak = 0
        for y in hm:
            if peak < y:
                peak = y
        ylim = float(peak) * 1.1
        fd.set_scale(min, 0, max, ylim)
        for i in range(numbins):
            xy = []
            xy.append( (min + i*binsize, 0))
            xy.append( (min+(i+1)*binsize, hm[i]))
            fd.box(xy)


def sample():
    global fd, data, index, result
    i = int(function.get())
    cmd = func_list[i]
    p1 = src.get()
    p2 = dst.get()
    if cmd == 'r2rtime':
      t  = fd.r2rtime(p1,p2)
    elif cmd == 'r2ftime':
      t  = fd.r2ftime(p1,p2)
    elif cmd == 'f2rtime':
      t  = fd.f2rtime(p1,p2)
    elif cmd == 'f2ftime':
      t  = fd.f2ftime(p1,p2)
    elif cmd == 'set2rtime':
      t  = fd.set2rtime(p1,p2)
    elif cmd == 'set2ftime':
      t  = fd.set2ftime(p1,p2)
    elif cmd == 'clr2rtime':
      t  = fd.clr2rtime(p1,p2)
    elif cmd == 'clr2ftime':
      t  = fd.clr2ftime(p1,p2)
    elif cmd == 'pulse2rtime':
      t  = fd.pulse2rtime(p1,p2, int(pwidth.get()),ppol.get())
    elif cmd == 'pulse2ftime':
      t  = fd.pulse2ftime(p1,p2, int(pwidth.get()),ppol.get())
    elif cmd == 'multi_r2rtime':
      t  = fd.multi_r2rtime(p1,0)
    elif cmd == 'pendulum_period':
      t  = fd.pendulum_period(p1)
    else:
      print 'not implemented'
      return
     
    if t < 0:
      print 'time out error: ',cmd
      s = 'time out error'
      result.config(text = s)
      return 
    s = str(t)
    print s
    result.config(text = s)
    mv = (index, float(t))
    data.append(mv)
    if len(data) > 1:
        fd.line(data)
    index = index + 1
    if index <= Npoints.get():
        root.after(5, sample)
  
def start():
  global fd, data, root, index
  index = 0
  data = []
  fd.remove_lines()
  np = Npoints.get()
  fd.remove_boxes()
  fd.set_scale(0, 0, np, int(tlim.get()))
  root.after(5, sample)

def save():
  global data,fd
  fd.save_data(data,'time.dat')

fd = phm.phm()
fd.set_adc_size(2)

root = Tk()
lv = Frame(root, padx = 5, pady = 5)
lv.pack(side=LEFT, fill = Y)

function = StringVar()
src = IntVar()
dst = IntVar()

l = Label(lv, text = 'Functions', bg = 'white')
l.pack(side = TOP, fill = X)

for i in range(len(func_list)):
    c = Radiobutton(lv, variable = function, val = i, text = func_list[i])
    c.pack(side=TOP, anchor = W, fill = Y)
function.set(0)

c = Frame(lv)
c.pack(side=TOP)
l = Label(c, text = 'I/O Socket', bg = 'white')
l.pack(side = TOP, fill = X)
c1 = Frame(c)
c1.pack(side=LEFT, anchor = N)
c2 = Frame(c)
c2.pack(side=RIGHT, anchor = N)

l = Label(c1, text = 'From')
l.pack(side = TOP, fill = X, expand = 1)
l = Label(c2, text = 'To')
l.pack(side = TOP, fill = X, expand = 1)

for i in range(len(src_list)):
    c = Radiobutton(c1, variable = src, val = i, text = src_list[i])
    c.pack(side=TOP, anchor=W)
src.set(0)

for i in range(len(dst_list)):
    c = Radiobutton(c2, variable = dst, val = i, text = dst_list[i])
    c.pack(side=TOP, anchor =W)
dst.set(0)


l = Label(lv, text = 'Pulse Width', bg = 'white')
l.pack(side = TOP, fill = X)

pwidth = StringVar()
l = Entry(lv, textvariable = pwidth, bg = 'white')
l.pack(side = TOP, fill = X)
pwidth.set(13)

ppol = IntVar()
l = Checkbutton(lv, variable = ppol, text = 'Pulse Polarity')
l.pack(side = TOP, fill = X)
ppol.set(0)

l = Label(lv, text = 'Time Limit (usec)', bg = 'white')
l.pack(side = TOP, fill = X)

tlim = StringVar()
l = Entry(lv, textvariable = tlim, bg = 'white')
l.pack(side = TOP, fill = X)
tlim.set(1000)

fd.window(300,200,root)
cmd = Frame(root)
cmd.pack(side=RIGHT, anchor = NE)

l = Label(cmd, text = 'N = ')
l.pack(side=LEFT)

Npoints = Scale(cmd, from_ = 5, to = 50, length=100, resolution = 5,\
orient = HORIZONTAL)
Npoints.pack(side=LEFT)


b = Button(cmd,text='Start', command = start)
b.pack(side=LEFT, anchor = S)

b = Button(cmd,text='Histogram', command = hist)
b.pack(side=LEFT, anchor = S)

b=Button(cmd,text='Save', command = save)
b.pack(side=LEFT, anchor = S)

result = Label(cmd, text = ' ', width = 20,bg = 'white')
result.pack(side=LEFT, anchor = S)

fd.set_scale(0, 0, Npoints.get(), 2.e6)	# 2 seconds maximum

fd.set_frequency(1000)

root.mainloop()
