import phm, time, math
from Tkinter import *

class disp:
    """
    Class for displaying items in a canvas using a global coordinate system.
    """
    border = 5
    bordcol = '#555555'
    gridcol = '#f0f0f0'
    gridcol2 ='#e0e0e0'
    def __init__(self, parent, width=400., height=300.):
	self.parent = parent
	self.SCX = width
	self.SCY = height
	self.XLIM = width + 2 * self.border
	self.YLIM = height + 2 * self.border
	self.canvas = Canvas(parent, background="white",	width = self.XLIM, height = self.YLIM)
	self.canvas.pack()
        b1 = (self.border - 1, self.border-1)
        b2 = (self.XLIM - self.border + 1, self.YLIM - self.border + 1)
	self.canvas.create_rectangle ([b1,b2], outline = self.bordcol)
	self.canvas.pack()
	self.setWorld(-0.5 * width, -0.5*height, 0.5 * width, 0.5* height)
	self.grid(10,100)
	self.canvas.bind("<Button-1>", self.show)
	self.canvas.bind("<Button-3>", self.show)
	self.xydisp = None
	self.marker = None

    def show(self,event):
        if self.xydisp != None:
          self.canvas.delete(self.xydisp)
        ix = self.canvas.canvasx(event.x) - self.border
        iy = self.YLIM - self.canvas.canvasy(event.y) - self.border
        x = float(ix) * self.xscale + self.xmin
        y = float(iy) * self.yscale + self.ymin
        if event.num == 1:
          s = 'x = %6.3f\ny = %6.0f' % (x/1000.,y)
          self.marker = (x,y)
        elif event.num == 3 and self.marker != None:
          s = 'x = %6.3f  dx = %6.3f\ny = %6.0f  dx = %6.0f' % \
          (self.marker[0]/1000, (x-self.marker[0])/1000., self.marker[1],\
          y - self.marker[1])
        self.xydisp = self.canvas.create_text(self.border+1,self.SCY-1, \
        anchor = SW, justify = LEFT, text = s)

    def setWorld(self, x1, y1, x2, y2):
      #Calculate the scale factors to be used by functions drawPoint etc.
      self.xmin = x1
      self.ymin = y1
      self.xmax = x2
      self.ymax = y2
      self.xscale = (self.xmax - self.xmin) / (self.SCX)
      self.yscale = (self.ymax - self.ymin) / (self.SCY)

    def w2s(self, p):
       ip = []
       for xy in p:
         ix = self.border + int( (xy[0] - self.xmin) / self.xscale)
         iy = self.border + int( (xy[1] - self.ymin) / self.yscale)
         iy = self.YLIM - iy
         ip.append((ix,iy))
       return ip

    def box(self, x1,  y1,  x2,  y2, col):
       ip = self.w2s([(x1,y1),(x2,y2)])
       self.canvas.create_rectangle(ip, outline=col)

    def line(self, points, col):
       ip = self.w2s(points)
       return self.canvas.create_line(ip, fill=col, smooth = 1)

    def delete_line(self, trace):
       self.canvas.delete(trace)

    def grid(self, major, minor):
       dx = (self.xmax - self.xmin) / major
       dy = (self.ymax - self.ymin) / major
       x = self.xmin + dx
       while x < self.xmax:
         self.line([(x,self.ymin),(x,self.ymax)],self.gridcol)
         x = x +dx
       y = self.ymin + dy
       while y < self.ymax:
         self.line([(self.xmin,y),(self.xmax,y)],self.gridcol)
         y = y +dy

       dx = (self.xmax - self.xmin) / minor
       dy = (self.ymax - self.ymin) / minor
       x = self.xmin + dx
       while x < self.xmax:
         self.line([(x, 0.),(x, dy)],self.gridcol2)
         x = x +dx

       y = self.ymin + dy
       while y < self.ymax:
         self.line([(0., y),(dx,y)],self.gridcol2)
         y = y +dy

class CRO:
	global fd
	NPMAX = 200
	NP = 200		# Number of points to be sampled
	width = 600		# Window X width, MUST be multiple of NP
	height = 512		# height, MUST be multiple of ADC range(256)
	xmin = 0.0
	xmax = 2000.0
	ymin = -5000.0
	ymax =  5000.0
	color = ['black', 'red', 'green', 'blue']

	MAXCHAN = 4
	numchans = 1
	chmask = 1			# Channel mask
	val = [10,20,50,100,200,500,1000]
	delay = None			# delay is IntVar() of set_delay() 
	
	root = None			# Tkinter Widgets

	chan_status = []
	data = []
	traces = None
	
	def __init__(self, root):
		self.parent = root
		mf = Frame(root)
		mf.pack(side=TOP)
		self.screen = disp(root,self.width, self.height)

		#Control widgets start from here
		row1 = Frame(root,  relief=RIDGE)
		row1.pack(side=TOP, ipady=10, ipadx= 5,anchor = W)
		self.trigpol = IntVar()
		self.Trigpol = Checkbutton(row1, text='+/- Tr',\
			selectcolor = 'yellow',	variable = self.trigpol, command = self.set_pol)
		self.Trigpol.pack(side = LEFT)

		self.Trig =  Scale(row1, command = self.set_trig, from_ = 5,\
		to=250, orient=HORIZONTAL, length=self.width/6, showvalue=0)
		self.Trig.set(125)
		self.Trig.pack(side=LEFT)
		
		l = Label(row1,text='CH ')
		l.pack(side=LEFT)
		for k in range(4):
			var = IntVar()
			c = Checkbutton(row1, variable = var, text=str(k+1),\
			fg =	self.color[k], selectcolor = self.color[k], \
			command = self.select_channels)
			self.chan_status.append(var)
			c.pack(side=LEFT)
		self.chan_status[0].set(1)
		
		self.Delay = Scale(row1, command = self.set_delay, from_ =0,\
		to=6, orient=HORIZONTAL, showvalue=0)
		self.Delay.pack(side=LEFT)
		self.Delay.set(0)
		
		self.timebase = StringVar()
		self.tblab = Label(row1, width = 10, textvariable = self.timebase)
		self.tblab.pack(side=LEFT)

		self.zoom = IntVar()
		self.Zoom = Checkbutton(row1, text='10x', variable = self.zoom,\
			onvalue = 10, offvalue = 1,\
			selectcolor = 'yellow', command = self.set_zoom)
		self.Zoom.pack(side = LEFT)

		self.Liz = IntVar()
		self.Zoom = Checkbutton(row1, text='LIZ', variable = self.Liz,\
			onvalue = 1, offvalue = 0, selectcolor = 'yellow')
		self.Zoom.pack(side = LEFT)
		self.Liz.set(0)
		
# Add Second ROW of widgets
		row2 = Frame(root)
		row2.pack(side=TOP, ipady=5, ipadx= 5,anchor = W)

		b = Button(row2, text='Freq',command = self.get_frequency)
		b.pack(side=LEFT)
		self.fchan = StringVar()
		for k in range(4):
			c = Radiobutton(row2, variable = self.fchan, value = k,\
			selectcolor = self.color[k])
			c.pack(side=LEFT)
		c = Radiobutton(row2, text='CNTR',variable = self.fchan, value = 4,\
			selectcolor = 'yellow')
		c.pack(side=LEFT)
		self.fchan.set(0)

		self.Freq = Label(row2, bg = 'white', width = 10)
		self.Freq.pack(side=LEFT)

		b = Button(row2, text = 'set PWG =', command = self.set_pwg)
		b.pack(side=LEFT)
		self.PWG = StringVar()
		pwg = Entry(row2, bg = 'white', width = 10,textvariable = self.PWG)
		pwg.pack(side=LEFT)

		self.Save = Button(row2, text='Save',command = self.save)
		self.Save.pack(side=LEFT)

		self.Print = Button(row2, text='Print',command = self.eps)
		self.Print.pack(side=LEFT)

		fd.set_num_samples(self.NP)
		fd.set_adc_size(1)
		self.select_channels()
		self.parent.after(10, self.draw)


	def set_pwg(self):
		f = int(self.PWG.get())
		if 0 < f < 4000000:
			sf = fd.set_frequency(f)
			s = '%6d' % (sf)
		else:
			s = 'set 0 to 4MHz' 
		
		self.PWG.set(s)
		
	def eps(self):
		self.screen.canvas.postscript(file = 'cro.eps', colormode = 'color')
		
	def save(self):
		fd.save_data(self.data, 'cro.dat') 

	def set_zoom(self):
		z = self.zoom.get()
		self.NP = self.NPMAX / z
		fd.set_num_samples(self.NP)
		self.set_delay(0)
			
	def set_trig(self, dummy):
		tr = self.Trig.get()
		pol = self.trigpol.get()
		if pol == 1:		# -ive edge trigger
			fd.set_adc_trig(tr,tr-5)
		else:
			fd.set_adc_trig(tr,tr+5)

	def set_pol(self):
		self.set_trig(0)
		
	def set_delay(self,dummy):
		d = self.Delay.get()
		self.delay = self.val[d]
		fd.set_adc_delay(self.delay)	
		perdiv = float(self.NP) * self.delay * self.numchans / 10000
		self.timebase.set(str(perdiv) +' ms/div')
		self.xmax = self.NP * self.delay
		self.screen.setWorld(self.xmin, self.ymin,self.xmax, self.ymax)

	def get_frequency(self):
		global start
		res = self.fchan.get()
		ch = ord(res)-ord('0')
		if ch == 4:
			fr = fd.measure_frequency()
			s = '%5.0f Hz'%(fr)
		else:
			t = fd.adc_input_period(ch)
			if t < 0:
				s = 'No signal'
			else:
				fr = 1000000.0/ t
				s = '%5.0f Hz'%(fr)
		self.Freq.config(text = s)

		sec = fd.get_time()-start
		s = '%d s'%(sec)
		self.Freq.config(text = s)

	def select_channels(self):
		self.chmask = 0
		self.numchans = 0
		for k in range(4):
			if self.chan_status[k].get() == 1:
				self.chmask = self.chmask | (1 << k)
				fd.add_channel(k)
				self.numchans = self.numchans  + 1
			else:
				fd.del_channel(k)
		self.set_delay(0)
		
	def draw(self):
		try:
			self.data = fd.multi_read_block(self.NP,self.delay,1)
		except:
			self.parent.after(10, self.draw)
			return
			
		if self.traces != None:
			for ch in range(len(self.traces)):
				self.screen.delete_line(self.traces[ch])
		self.traces = []
		
		mode = self.Liz.get()
		if (mode == 1) and (self.chmask == 3):
			self.screen.setWorld(self.ymin, self.ymin, self.ymax, self.ymax)
			points = []
			for n in range(self.NP):
				points.append((self.data[n][1], self.data[n][2]))
			t = self.screen.line(points, 'black')
			self.traces.append(t)
		else:
			self.screen.setWorld(self.xmin, self.ymin, self.xmax, self.ymax)
			ch = 0;
			for k in range(self.MAXCHAN):
				if self.chmask & (1 << k):	# Draw if selected channel
					points = [] 
					for n in range(self.NP):
						points.append((self.data[n][0],self.data[n][ch+1] ))
					t = self.screen.line(points, self.color[k])
					self.traces.append(t)
					ch = ch + 1
		self.parent.after(10, self.draw)

fd = phm.phm()
root = Tk()
CRO(root)
start = fd.set_time()

fd.clear_plugins()
v = []
for i in range(100):
    x = 127.5 + 127.5 * math.sin(2.0*math.pi*i/100)
    x = int(x+0.5)
    v.append(x)

#for i in range(50):
#    v.append(128)    

fd.load_wavetable(v)	
print fd.start_wave(100,'hruser')

root.mainloop()
